import socket
from loguru import logger

from qldev.device import MessageHandler, MessageType

class TCPServer(object):
    def __init__(self, port, callback=None) :
        self.__port = port
        self._callback = callback
        self._run = False
        self._socket = None
        self.clients = {}
        self._start()

    def is_run(self):
        return self._run
    
    def exists(self, devno):
        return not self.get_client(devno)
    
    def accept(self, handler : MessageHandler = None):

        while self._run:
            # 等待新的客户端连接
            client_socket, client_address= self._socket.accept()
            logger.info(f"收到新的设备连接: {client_address}")
            if handler:
                handler.accept(client_socket, client_address, MessageType.SOCKET)

    def client_handler(self, socket, addr):
        logger.info(f"new client socket {addr[0]}:{addr[1]} connected.")


    def _start(self):
        if self.is_run():
            return

        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        # 本地信息
        address = ('', self.__port)
        # 绑定
        self._socket.bind(address)

        self._socket.listen(128)
        logger.info(f'开始监听端口{self.__port}的TCP消息')

        self._run = True
    
    def get_client(self, devno = None):   
        if len(self.clients) == 0:
            logger.warning("TCP server has no client connection.")
            return None

        if devno:
            return self.clients[devno]
        else:
            # 默认返回一个设备
            key = random.sample(self.clients.keys(), 1)[0]
            return self.clients[key]

        
def client_listen(socket, callback):
    while True:
        try:
            # 接收对方发送过来的数据
            recv_data = socket.recv(1024)  # 接收1024个字节
            if recv_data:
                msg = recv_data.decode('utf-8')
                logger.debug(f'receive -> {msg}')
                callback     
        except Exception as e:
            logger.error("Exception on client_listen")
            logger.error(e)
            break

class ServerContainer(object):
    def __init__(self) :
        self.servers = {}
        
    def add_listen(self, port):
        if self.exists(port):
            logger.warning(f"server[{port}] exists already.")
            return
        tcp_server = TCPServer(port)
        self.servers[port] = tcp_server

        return tcp_server

        
    def exists(self, port):
        if self.servers[port]:
            return True
        return False




import os
import socket

# 读取本机全部ip列表
# return list
def get_ips():
    return socket.gethostbyname_ex(socket.gethostname())[-1]

# 读取本机的ip地址
# return str
def get_ip():
    # 优先读取活跃ip地址
    routes = os.popen('route print').readlines()
    for idx, item in enumerate(routes):
        if ' 0.0.0.0 ' in item and len(item.split()) > 2:
            return item.split()[-2]
    
    # 取第一个地址
    ips = get_ips()
    if len(ips) > 0 :
        return ips[0]

    raise ValueError("Ip address not exists.")



import enum

class MessageType(enum.Enum):
    UDP = 'udp'
    TCP = 'tcp'
    SOCKET = 'socket'

class MessageParser(object):
    def __init__(self) -> None:
        pass

class MessageHandler(object):
    def __init__(self) -> None:
        print('MessageHandler __init__')

    def accept(self, message, address, type : MessageType = None):
        pass

    def add_client(self, client):
        pass

    def remove_client(self, client):
        pass

    


from time import sleep
from qldev.network import UDPProxy, TCPServer
from .client import *
from .handler import *
from .model import *

# 盒子服务端
class BoxServer(object): 
    def __init__(self, auto_search = False) -> None:
        self.__auto_search = auto_search
        self.__handler = BoxMessageHandler()
        self.__udp_port = 54366
        self.__tcp_port = 19128
        self.__udp_proxy = None
        self.__tcp_server = None
        self.__searching = False

    @property
    def clients(self):
        if self.__handler:
            return self.__handler.clients

        return None

    @property
    def broadcast_list(self):
        if self.__handler and self.__handler.broadcast_list:
            return self.__handler.broadcast_list.difference(self.__handler.connected_list)

        return None

    @property
    def connected_list(self):
        if self.__handler:
            return self.__handler.connected_list

        return None

    def start(self, udp_port = None, tcp_port = None):
        if udp_port:
            self.__udp_port = udp_port
        if tcp_port:
            self.__tcp_port = tcp_port
        
        self.__udp_listen()
        self.__tcp_listen()
        self.start_search()

        return self

    def __udp_listen(self):
        if self.__udp_proxy is None:
            self.__udp_proxy = UDPProxy(self.__udp_port)

        if self.__auto_search:
            self.__client_detect()

    # UDP消息监听-设备探测
    # 子线程
    def __client_detect(self):
        detect_thread = Thread(target=self.__udp_proxy.accept, args=(self.__udp_port, 512, self.__handler))
        detect_thread.setDaemon(True)
        detect_thread.start()

    def __tcp_listen(self):
        if self.__tcp_server is None:
            self.__tcp_server = TCPServer(self.__tcp_port)
        
        self.__client_accept()

    # 客户端连接请求
    # 子线程
    def __client_accept(self):
        accept_thread = Thread(target=self.__tcp_server.accept, args = (self.__handler,))
        accept_thread.setDaemon(True)
        accept_thread.start()

    # UDP消息发送-设备连接通知
    # 子线程
    def start_search(self, repeat = True):
        if self.__searching:
            logger.warning("重复的指令，搜寻设备任务已启动!")
            return
        self.__searching = True
        accept_thread = Thread(target=self.__search_all, args=(repeat,))
        accept_thread.setDaemon(True)
        accept_thread.start()

    def __search_all(self, repeat):
        while repeat and self.__searching:     
            if self.broadcast_list is None or len(self.broadcast_list) == 0:
                continue

            for idx, item in enumerate(self.broadcast_list):
                logger.debug(f"搜寻设备：{item}")
                try:
                    message = UDPMessage.get_message(item, tcp_port = self.__tcp_port)            
                    self.__udp_proxy.broadcast(message=message)
                except Exception as e:
                    logger.error(f"搜寻设备消息发送异常: {e}")
            # 广播间歇
            sleep(2)
        
    def add_search(self, devno):
        if devno:
            self.__handler.add_search(devno)

    # 获取可用的客户端（设备）
    def get_client(self, mac = None) -> BoxClient:
        if mac and mac in self.clients.keys():
            return self.clients[mac]
        elif len(self.clients) > 0:
            return list(self.clients.values())[0]
        else:
            return None

    def subscribe(self, callback, key = 'default', mac = None):
        client = self.get_client(mac)
        if client:
            consumer = BoxMessageQueue()
            client.subscribe(consumer = consumer, key = key)
            t = Thread(target=self.__consumer, args=(consumer, callback))
            t.setDaemon(True)
            t.start()
        else:
            raise Exception('没有可用的设备')

    def __consumer(self, consumer:BoxMessageQueue, callback):
        while True:
            if consumer.empty():
                sleep(0.1) ## sleep 100ms 
            else:
                callback(consumer.get())


import enum
from queue import Queue
import time
import numpy as np
from loguru import logger

class Acc(enum.Enum):
    X = 0
    Y = 1
    Z = 2

class BoxPacket(object):
    def __init__(self, id, points) -> None:
        self.__id = id
        self.__eeg, self.__acc = BoxPacket.parse(points)
        
    def __lt__(self, other):
        return self.__id < other.__id

    def __gt__(self, other):
        return self.__id > other.__id

    def __eq__(self, other):
        return self.__id == other.__id

    @property
    def id(self):
        return self.__id

    # 一维数组
    @property
    def eeg(self):
        return self.__eeg

    # 一维数组/二维数组
    @property
    def acc(self, dim : Acc = None):
        if dim :
            return self.__acc[dim.value]
        
        return self.__acc
        
    @staticmethod
    def parse(data):
        # 1, 2 数据相同（只有一通道数据)
        eegs = []
        # x, y, z
        acc = [[],[],[]]
        for i in range(0, 200, 4):
            eegs.append(int.from_bytes(data[i : i + 2], 'little'))
        for i in range(200, 230, 6):
            acc[0].append(int.from_bytes(data[i : i + 2], 'little'))
            acc[1].append(int.from_bytes(data[i + 2 : i + 4], 'little'))
            acc[2].append(int.from_bytes(data[i + 4 : i + 6], 'little'))

        # 数字信号转换为电压值(mv)
        eegs = (np.array(eegs) -32768) * (2500000 / 48 / 4 / 32768)
        return eegs, acc

class BoxMessageQueue(Queue):
    def __init__(self, maxsize: int = 0) -> None:
        super().__init__(maxsize)
        self.__uid = int(time.time() * 1000000)
        self.__latest_id = None
        self.__total = 0

    def put(self, v:BoxPacket):
        super().put(v)
        self.__latest_id = v.id
        self.__total += 1
        if self.qsize() >= 3000 and self.qsize() % 1000 == 0:
            logger.warning(f'队列（{self.__uid}）已有{self.__total}条数据未消费，请关注。')

    @classmethod
    def unique_id(self, key):
        self.__uid = key

    @property
    def latest_id(self):
        return self.__latest_id

    @property
    def total(self):
        return self.__total


from .handler import *

class Device(object):
    def __init__(self) -> None:
        pass

class QLDevice(Device):
    def __init__(self, handler: MessageHandler = None) -> None:
        super().__init__()
        self.__handler = handler

    @property
    def handler(self):
        return self.__handler


from cmath import log
from ipaddress import ip_address
from socket import *
from time import strftime
from loguru import logger

from qldev.device import MessageParser
from qldev.device.handler import MessageHandler, MessageType

class UDPSender(object):
    def __init__(self, ip = "255.255.255.255", port = 54366) :
        self._address = (ip, port)
        self._socket = None
        self.socket()
    
    def socket(self):
        if self._socket is None:
            self._socket = socket(AF_INET, SOCK_DGRAM)
            self._socket.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
        
        return self._socket
        
    def send(self, message):
        try:
            self._socket.sendto(message, self._address)
        except Exception as e:
            logger.error("Exception on UDPSender send.")
            logger.error(e)
    
    def close(self):
        if self._socket :
            self._close()


class UDPProxy(object):
    def __init__(self, port=54366):
        self.__port = port
        self.__Listening = {}
        self.__socket = socket(AF_INET, SOCK_DGRAM)

    def close(self):
        if self.__socket:
            self.__socket.close()
            logger.info(f"关闭端口{self.__port}的连接")
        else:
            logger.warning(f"未监听端口{self.__port}的UDP消息")
    
    def accept(self, port: None, bufsize: int = 1024, handler:MessageHandler = None):
        if port is None:
            port = self.__port
        if port is None:
            logger.error(f"未设置UDP消息监听端口！")
            return
        if port in self.__Listening.keys():
            logger.warning(f"请不要重复监听端口{port}消息")

        self.__socket.bind(('', port)) 
        logger.debug(f"开始监听端口({port})的UDP消息")
        self.__Listening[port] = True

        while self.__Listening[port]:
            buffer = self.__socket.recvfrom(bufsize) 
            if buffer and handler:
                handler.accept(buffer[0], buffer[1], MessageType.UDP)
        
        self.__Listening[port] = False
        
    def broadcast(self, message, port = None):
        if port is None:
            port = self.__port
        address = ('255.255.255.255', port)
        self.__socket.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)

        try:
            self.__socket.sendto(message, address)
        except Exception as e:
            logger.error("发送UDP消息异常：")
            logger.error(e)

        



from .crcutils import *
from .local import *

from enum import Enum 

class UDPCommand(Enum):
    NOTIFY = 0x9
    CONNECT = 0x10


class TCPCommand(Enum):
    GET_DEVICE_INFO = 0x10
    SIGNALS = 0X455
    TRIGGER = 0x501
    WAV_UPDATE_READY = 0X502
    WAV_UPDATE = 0X503
    WAV_UPDATE_FINISH = 0X504
    WAV_TRIGGER = 0X505



from email.policy import default
import hashlib
from queue import Queue
from signal import raise_signal, signal
from threading import Lock, Thread
from tkinter.tix import Tree
from loguru import logger
import time 

from qldev.box.message import TCPMessage
from qldev.box.model import BoxMessageQueue, BoxPacket
from qldev.device import MessageHandler, QLDevice
from qldev.utils import TCPCommand


class BoxClient(QLDevice):
    ID = 'mac'
    def __init__(self, socket, address = None, handler: MessageHandler = None) -> None:
        super().__init__(handler)
        self.__socket = socket
        self.__address = address
        self.__parser = BoxClientMessage(self)
        self.__create_time = int(time.time())
        self.__listening = False
        self.__info = None
        self.__channels = {}

    @property
    def mac(self):
        if self.__info and BoxClient.ID in self.__info.keys():
            return self.__info[BoxClient.ID]
        return None

    @property
    def address(self):
        return self.__address

    @property
    def key(self):
        if self.__address and len(self.__address) == 2:
            return f'{self.__address[0]}_{self.__address[1]}'
        return None

    def subscribe(self, consumer:BoxMessageQueue, key = 'default'):
        if key is None:
            key = 'default'
        
        if key in self.__channels.keys():
            logger.error(f'subscribe({key}) already exists in device({self.__address}). ')
            return 

        if consumer:
            consumer.unique_id(f'{self.mac}_{key}')
            self.__channels[key] = consumer
        logger.info(f'已添加设备device({self.mac} - {self.__address}的消息订阅（{key}），当前共有订阅任务{len(self.__channels)}个。')

    def listen(self, buffersize = 1024):
        if self.__listening:
            logger.warning(f"BoxClient({self.mac}-{self.__address}) is listening.")
            return

        t = Thread(target=self.__accept, args=(buffersize, ))
        t.setDaemon(True)
        t.start()

        return self

    def update(self, devinfo):
        logger.trace(devinfo)
        if devinfo :
            self.__info = devinfo
            if self.mac:
                self.handler.add_client(self)

    # 信号数据
    def signals(self, data):
        if self.mac is None or len(self.__channels) == 0:
            return

        logger.trace(data)
        for key in self.__channels.keys():
            try:
                self.__channels[key].put(data)
            except Exception as e:
                logger.error(f"BoxClient({self.mac} - {self.__address}) 添加消息到订阅（{key}）异常: {e}")


    def ask(self):
        self._send(TCPMessage.get_device_info())
        
        return self
        
    def trigger(self, type=0):
        self._send(TCPMessage.trigger(type))

    def wav_update_ready(self, id, len):
        self._send(TCPMessage.wav_update_ready(id, len))

    def wav_update(self, id, offset, len, data):
        self._send(TCPMessage.wav_update(id, offset, len, data))

    def wav_update_stop(self, id, checksum):
        self._send(TCPMessage.wav_update_stop(id, checksum))

    def wav_trigger(self, id, volumn):
        self._send(TCPMessage.wav_trigger(id, volumn))
       
    # id是音频编号，取值范围为[0-7]，表示第1-8个音频位置
    # fname是本地音频文件的存储路径  
    def update_wav(self, id, fname, buffer=3072):    

        with open(fname, 'rb') as f:
            flen = f.seek(0, 2)
            logger.debug(f"file{fname} len is {flen}")
            self.wav_update_ready(id, flen)
            time.sleep(0.1)

            f.seek(0)
            offset = 0
            buf = f.read(buffer)
            md5 = hashlib.md5()
            while buf:
                blen = len(buf)
                logger.debug(f"update(id:{id}, offset:{offset}/{flen}, llen:{blen})")
                self.wav_update(id, offset, blen, buf)
                md5.update(buf)
                # 指令发送间隔，避免数据处理异常
                time.sleep(0.01)
                offset += blen
                buf = f.read(buffer)
            logger.debug(f"file{fname} offset is {offset} md5 is {md5.hexdigest()}")

            self.wav_update_stop(id, md5.hexdigest())
            logger.debug(f'file{fname} update finished.')    

    def __accept(self, buffersize = 1024):
        self.__listening = True
        while self.__listening:
            try:
                # 接收对方发送过来的数据
                recv_data = self.__socket.recv(buffersize)  
                if recv_data:                    
                    logger.trace(f'receive len is {len(recv_data)}')
                    self.__parser.extend(recv_data)
                    self.__parser.parse()
                else:
                    break
            except Exception as e:
                logger.error(f"BoxClient({self.mac}-{self.__address})接收消息出现异常：{e}")
                break

        self.__socket.close()
        logger.info(f"BoxClient({self.mac}-{self.__address})连接已断开")

        # 释放当前实例
        self.__release()

    def _send(self, message):
        try:
            self.__socket.send(message)
        except Exception as e:
            logger.warning(f"发送TCP消息异常:{e}")

    def __release(self):
        self.__parser = None
        self.handler.remove_client(self)


class BoxClientMessage(object):
    # 包头
    HEADER = 42330

    def __init__(self, client : BoxClient) -> None:
        self.__client = client
        self.__messages = bytearray()
        self.__lock = Lock()

    def extend(self, message):
        if len(message) > 0:
            self.__lock.acquire()
            self.__messages.extend(message)
            self.__lock.release()

    def parse(self):        

        while len(self.__messages) > 12:
            self.__lock.acquire()
            while len(self.__messages) > 12 and int.from_bytes(self.__messages[:2], "little") != BoxClientMessage.HEADER:
                self.__messages.pop(0)
            self.__lock.release()

            mlen = int.from_bytes(self.__messages[8:12], "little")
            logger.trace(f'The current packet length is {mlen}')

            if mlen > 0 and len(self.__messages) >= mlen:
                packet = self.__messages[:mlen]
                self.__lock.acquire()
                self.__messages = self.__messages[mlen:]
                self.__lock.release()
                self.__parse(packet)
            else:
                break
        
    def __parse(self, packet):
        plen = len(packet)
        # 最小有效包长度
        if plen < 16:
            logger.warning(f"不支持的消息长度: {len}")
            return
        
        cmd = int.from_bytes(packet[12:14], 'little')
        data = packet[14:-2]
        result = (int)(data[8])
        if result != 0:
            logger.warning(f"cmd {hex(cmd)} result is {result}(0: success others: failed)")
            return

        if cmd == TCPCommand.GET_DEVICE_INFO.value:
            devinfo = self.get_device_info(data)
            self.__client.update(devinfo)
            return
            
        if cmd == TCPCommand.SIGNALS.value:
            self.__client.signals(self.signals(data))
            return
            
        if cmd == TCPCommand.TRIGGER.value:
            self.trigger(data)
            return
            
        if cmd == TCPCommand.WAV_UPDATE_READY.value:
            self.wav_update_ready(data)
            return
            
        if cmd == TCPCommand.WAV_UPDATE.value:
            self.wav_update(data)
            return
            
        if cmd == TCPCommand.WAV_UPDATE_FINISH.value:
            self.wav_update_stop(data)
            return
            
        if cmd == TCPCommand.WAV_TRIGGER.value:
            self.wav_trigger(data)
            return

        self.not_support(cmd, data)

    def get_device_info(self, data):
        info = {}
        info["mac"] = hex(int.from_bytes(data[9:17], 'little'))
        info["dev_type"] = int.from_bytes(data[17:21], 'little')
        info["sw_version"] = int.from_bytes(data[21:25], 'little')
        info["hw_version"] = int.from_bytes(data[25:29], 'little')
        logger.debug(f"get_device_info->{info}")
        return info

    def signals(self, data):
        logger.trace(f"The signal data length is {len(data)}")
        # packet = {}
        # packet["packet_id"] = int.from_bytes(data[9:17], 'little')
        # packet["channels"] = data[17:49]
        # packet["origin_rate"] = int.from_bytes(data[49:53], 'little')
        # packet["sample_rate"] = int.from_bytes(data[53:57], 'little')
        # packet["sample_len"] = int.from_bytes(data[57:61], 'little')
        # packet["resolution"] = int.from_bytes(data[61:63], 'little')
        # packet["filter"] = int(data[63])
        # packet["data_len"] = int.from_bytes(data[64:68], 'little')
        # packet["data"] = data[68:]
        id = int.from_bytes(data[9:17], 'little')
        points = data[68:]

        return BoxPacket(id, points)
        
    def trigger(self, data):
        logger.debug("trigger success.")
        return
        
    def wav_update_ready(self, data):
        logger.debug("wav_update_ready success.")
        return
        
    def wav_update(self, data):
        logger.debug("wav_update success.")
        return
        
    def wav_update_stop(self, data):
        logger.debug("wav_update_finsih success.")
        return
        
    def wav_trigger(self, data):
        logger.debug("wav_trigger success.")
        return
        
    def not_support(self, cmd, data):
        logger.warning(f"cmd({hex(cmd)})not support.")
        return



from loguru import logger
from qldev.box.client import BoxClient
from qldev.box.message import UDPMessage

from qldev.device import MessageHandler, MessageType

class BoxMessageHandler(MessageHandler):
    def __init__(self) -> None:
        super().__init__()
        self.clients = {}
        self.__broadcast_list = set([])
        self.__connected_list = set([])
        self.__temp = {}

    @property
    def broadcast_list(self):
        return self.__broadcast_list

    @property
    def connected_list(self):
        return self.__connected_list

    def accept(self, message, address, type : MessageType = MessageType.TCP):
        logger.trace(f"new {type.value} message received.")
        if type == MessageType.UDP:
            devinfo = UDPMessage.parse(message)
            logger.trace(devinfo)
            if devinfo and UDPMessage.MAC in devinfo.keys():
                self.add_search(devinfo[UDPMessage.MAC])

            return

        if type == MessageType.SOCKET:
            self.__add_socket(message, address)
            return

        if type == MessageType.TCP:
            return

    def __add_socket(self, socket, address):
        logger.trace(socket)
        logger.trace(address)
        key = f'{address[0]}_{address[1]}'
        client = BoxClient(socket, address, self)
        self.__temp[key] = client
        # 启动消息处理，并发送消息问询设备信息
        client.listen().ask()

    def add_client(self, client):
        
        logger.info(f"添加设备到客户端列表（{client.mac} - {client.address}）")

        if client.mac:
            self.clients[client.mac] = client
            self.connected_list.add(client.mac)
            logger.info(f"添加完成，客户端列表当前设备数量：{len(self.clients)}")


    def remove_client(self, client):
        logger.info(f"从客户端列表中移除设备（{client.mac} - {client.address}）")
        if client.mac and client.mac in self.clients.keys():
            self.clients.pop(client.mac)
            self.connected_list.remove(client.mac)
            logger.info(f"移除完成，客户端列表当前设备数量：{len(self.clients)}")
        if client.key and client.key in self.clients.keys():
            self.__temp.pop(client.key)

        # 删除对象
        del client
        

    def add_search(self, devno):
        if devno and not (devno in self.__broadcast_list):
            self.__broadcast_list.add(devno)
            logger.info(f"发现新设备（{devno}）并加入搜寻列表")


from setuptools import setup
from setuptools import find_packages

VERSION = '1.1.0'
AUTHOR='eegion'
EMAIL='hehuajun@eegion.com'

option = {
    "build_exe": {
        "excludes":["test", "main"],
        'packages':['utils','network','device','box']
    }
}

setup(
    name='qldev',  # package name
    version=VERSION,  # package version
    author=AUTHOR,
    author_email=EMAIL,
    description='api for quanlan box since v1.2.2',  # package description
    packages=find_packages(),
    install_requires=['loguru', 'numpy'],
    package_data={
        "":["*.txt", "*.md"]
    },
    zip_safe=False
)



from loguru import logger

from qldev.utils import TCPCommand, UDPCommand
from qldev.utils import check_crc, crc16, get_ip


class UDPMessage(object):
    START = "SHQuanLan"
    MAC = 'mac'
    DEV_TYPE = "dev_type"
    VERSION_CODE = "version_code"
    VERSION_NAME = "version_name"

    def __init__(self) -> None:
        self._base = None
        self._message = None

    @staticmethod
    def parse(packet, address = None):
        plen = len(packet)
        if plen < 10:
            logger.trace("message length too short.")
            return

        start = packet[:9].decode("utf-8")
        # quanlan udp message
        if start != UDPMessage.START:
            return

        if not check_crc(packet):
            logger.warn(f"数据CRC校验失败，丢弃！")
            return
        
        # message command
        cmd = int.from_bytes(packet[10:12], 'little')

        return UDPMessage._parse(cmd, packet[12:])

    @staticmethod
    def _parse(cmd, data):
        # 只解析0x09
        if cmd == UDPCommand.NOTIFY.value:
            return UDPMessage.parseDeviceInfo(data)
        else:
            logger.trace(f'不支持的消息. cmd: {hex(cmd)} dlen: {len(data)} data: {data}')
            return None
        
    @staticmethod
    def parseDeviceInfo(data):
        devinfo = {}
        try:
            devinfo[UDPMessage.DEV_TYPE] = hex(int.from_bytes(data[:2], 'little'))
            devinfo[UDPMessage.MAC] = hex(int.from_bytes(data[2:10], 'little'))
            devinfo[UDPMessage.VERSION_CODE] = int.from_bytes(data[42:46], 'little')
            devinfo[UDPMessage.VERSION_NAME] = str(data[10:42],'utf-8').split('\x00')[0]
        except Exception as e:
            logger.error(f"parseDeviceInfo异常：{e}")

        return devinfo

    #devno是hex字符串
    @staticmethod
    def get_message(devno, tcp_port=19128):
        message = bytearray(28)
        message[:10] = UDPMessage.START.encode('utf-8')
        cmd = UDPCommand.CONNECT.value
        devtype = 0x0        
        # 本机ip
        ip = get_ip().split(".")
        
        message[10:12] = cmd.to_bytes(2, 'little')
        message[12:14] = devtype.to_bytes(2, 'little')
        message[14:22] = int(devno, 16).to_bytes(8, 'little')
        message[22] = (int)(ip[0])
        message[23] = (int)(ip[1])
        message[24] = (int)(ip[2])
        message[25] = (int)(ip[3])
        message[26:28] = tcp_port.to_bytes(2, 'little')
        checksum = crc16(message)

        return message + checksum.to_bytes(2, 'little')

class TCPMessage(object):
    start = 0xa55a

    @staticmethod
    def header(packet_type=2, dev_type=0, dev_id = 0):
        header = bytearray(12)
        header[:2] = TCPMessage.start.to_bytes(2, 'little')
        header[2] = packet_type
        header[3] = 42
        header[4:8] = dev_id.to_bytes(4, 'little')
        header[8:12] = int(16).to_bytes(4, 'little')

        return header

    @staticmethod
    def wrap(cmd, data=None):
        header = TCPMessage.header()
        if data:
            dlen = len(data) + 16
            header[8:12] = dlen.to_bytes(4, 'little')
            message = header + cmd.to_bytes(2, 'little') + data 
        else: 
            message = header + cmd.to_bytes(2, 'little')
        
        checksum = crc16(message)
        return message + checksum.to_bytes(2, 'little')
    
    @staticmethod
    def get_device_info():
        return TCPMessage.wrap(TCPCommand.GET_DEVICE_INFO.value)

    @staticmethod
    def trigger(type = 0):
        data = type.to_bytes(4, 'little')
        return TCPMessage.wrap(TCPCommand.TRIGGER.value, data)

    @staticmethod
    def wav_update_ready(id, len):
        data = bytearray(8)
        data[:4] = id.to_bytes(4, 'little')
        data[4:] = len.to_bytes(4, 'little')
        return TCPMessage.wrap(TCPCommand.WAV_UPDATE_READY.value, data)

    @staticmethod
    def wav_update(id, offset, len, content):
        data = bytearray(len + 12)
        data[:4] = id.to_bytes(4, 'little')
        data[4:8] = offset.to_bytes(4, 'little')
        data[8:12] = len.to_bytes(4, 'little')
        data[12:] = content
        return TCPMessage.wrap(TCPCommand.WAV_UPDATE.value, data)

    @staticmethod
    def wav_update_stop(id, md5sum):
        data = id.to_bytes(4, 'little')
        data += md5sum.encode("utf-8") + b'0x0' # 结尾符号解析
        return TCPMessage.wrap(TCPCommand.WAV_UPDATE_FINISH.value, data)

    @staticmethod
    def wav_trigger(id = 0, volumn = 2):
        data = bytearray(8)
        data[:4] = id.to_bytes(4, 'little')
        data[4:] = volumn.to_bytes(4, 'little')
        return TCPMessage.wrap(TCPCommand.WAV_TRIGGER.value, data) 